@@ -1,6 +1,10 @@
+from __future__ import unicode_literals
+
 from django.conf import settings
 from django.views.debug import get_safe_settings
 from django.utils.translation import ugettext_lazy as _
+from django.utils.datastructures import SortedDict
+
 from debug_toolbar.panels import DebugPanel
 
 
@@ -11,17 +15,17 @@ class SettingsVarsDebugPanel(DebugPanel):
     name = 'SettingsVars'
     template = 'debug_toolbar/panels/settings_vars.html'
     has_content = True
-    
+
     def nav_title(self):
         return _('Settings')
-    
+
     def title(self):
         return _('Settings from <code>%s</code>') % settings.SETTINGS_MODULE
-    
+
     def url(self):
         return ''
-    
+
     def process_response(self, request, response):
         self.record_stats({
-            'settings': get_safe_settings(),
+            'settings': SortedDict(sorted(get_safe_settings().items(), key=lambda s: s[0])),
         })
